/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.shared.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Base class for items in a UserComponent
 */
public class CallComponentItem implements Serializable {

    /** The id. */
    private double id;
        
    /** The tool tip. */
    private String toolTip;
    
    /** The change listeners. */
    private List<CallComponentItemChangeListener> changeListeners = new ArrayList<CallComponentItemChangeListener>();

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /**
     * Instantiates a new call component item.
     */
    public CallComponentItem() {
        id = Math.random();
    }

    /**
     * Gets the id.
     *
     * @return the id
     */
    public double getId() {
        return id;
    }

    /**
     * Sets the id.
     *
     * @param id the new id
     */
    public void setId(double id) {
        this.id = id;
    }

    /**
     * Gets the tool tip.
     *
     * @return the tool tip
     */
    public String getToolTip() {
        return toolTip;
    }

    /**
     * Sets the tool tip.
     *
     * @param toolTip the new tool tip
     */
    public void setToolTip(String toolTip) {
        this.toolTip = toolTip;
        fireChangeListeners();
    }

    /**
     * Adds a change listener.
     *
     * @param listener the listener
     */
    public void addChangeListener(CallComponentItemChangeListener listener) {
        changeListeners.add(listener);
    }

    /**
     * Remove a change listener.
     *
     * @param listener the listener
     */
    public void removeChangeListener(CallComponentItemChangeListener listener) {
        changeListeners.remove(listener);
    }

    /**
     * Gets the change listeners.
     *
     * @return the change listeners
     */
    public List<CallComponentItemChangeListener> getChangeListeners() {
        return changeListeners;
    }

    /**
     * Fire change listeners.
     */
    protected void fireChangeListeners() {
        for (CallComponentItemChangeListener listener : changeListeners) {
            listener.callComponentItemChanged(new CallComponentItemChangedEvent(
                    this));
        }
    }

    /**
     * The listener interface for receiving callComponentItemChange events.
     * The class that is interested in processing a callComponentItemChange
     * event implements this interface, and the object created
     * with that class is registered with a component using the
     * component's <code>addCallComponentItemChangeListener<code> method. When
     * the callComponentItemChange event occurs, that object's appropriate
     * method is invoked.
     *
     * @see UserComponentItemChangeEvent
     */
    public static interface CallComponentItemChangeListener {

        /**
         * User component item changed.
         *
         * @param event the event
         */
        public void callComponentItemChanged(CallComponentItemChangedEvent event);

    }

    /**
     * Event fired when the call component item changes.
     * 
     * This is used to inform the CallComponent that one
     * of its items has changed and an event needs to be sent
     * to the client or the state needs to be marked as dirty.
     */
    public static class CallComponentItemChangedEvent {

        /** The item. */
        private CallComponentItem item;

        /**
         * Instantiates a new call component item changed event.
         *
         * @param item the item that has changed
         */
        public CallComponentItemChangedEvent(CallComponentItem item) {
            this.item = item;
        }

        /**
         * Gets the item that has changed.
         *
         * @return the item that has changed
         */
        public CallComponentItem getItem() {
            return item;
        }

        /**
         * Sets the item that has changed.
         *
         * @param item the item that has changed
         */
        public void setItem(CallComponentItem item) {
            this.item = item;
        }

    }

}
