package com.xmlnamespace.panel.vaadin.ui;

import java.util.HashSet;
import java.util.Set;

import com.vaadin.server.Resource;
import com.vaadin.server.ResourceReference;
import com.vaadin.shared.communication.URLReference;
import com.vaadin.ui.AbstractComponent;
import com.xmlnamespace.panel.vaadin.shared.ui.CallHistoryComponentRpc;
import com.xmlnamespace.panel.vaadin.shared.ui.CallHistoryComponentState;

public class CallHistoryComponent extends AbstractComponent {

    private static final long serialVersionUID = -6784020761261565421L;

    private InternalCallHistoryComponentRpc rpc = new InternalCallHistoryComponentRpc();

    private Set<DoubleClickListener> listeners = new HashSet<DoubleClickListener>();

    public CallHistoryComponent() {
        setWidth("100%");
        registerRpc(rpc);
    }

    @Override
    public void setIcon(Resource icon) {
        if (icon == null) {
            getState().icon = null;
        } else {
            getState().icon = new ResourceReference(icon, this, "ch-icon");
        }
        markAsDirty();
    }

    @Override
    public Resource getIcon() {
        URLReference ref = getState().icon;
        if (ref == null) {
            return null;
        }

        return ((ResourceReference) ref).getResource();
    }

    public String getCallerId() {
        return getState().callerId;
    }

    public void setCallerId(String callerId) {
        getState().callerId = callerId;
        markAsDirty();
    }

    public String getDateTime() {
        return getState().dateTime;
    }

    public void setDateTime(String dateTime) {
        getState().dateTime = dateTime;
        markAsDirty();
    }

    public String getCallLength() {
        return getState().length;
    }

    public void setCallLength(String callLength) {
        getState().length = callLength;
        markAsDirty();
    }

    public String getCallAccount() {
        return getState().account;
    }

    public void setCallAccount(String callAccount) {
        getState().account = callAccount;
        markAsDirty();
    }

    public void addDoubleClickListener(final DoubleClickListener listener) {
        listeners.add(listener);
    }

    public void removeDoubleClickListener(final DoubleClickListener listener) {
        listeners.remove(listener);
    }

    @Override
    public CallHistoryComponentState getState() {
        return (CallHistoryComponentState) super.getState();
    }

    public static interface DoubleClickListener {
        public void onDoubleClick();
    }

    private class InternalCallHistoryComponentRpc implements
            CallHistoryComponentRpc {

        private static final long serialVersionUID = 1L;

        /*
         * (non-Javadoc)
         * 
         * @see com.xmlnamespace.panel.vaadin.shared.ui.CallHistoryComponentRpc#
         * onDoubleClick()
         */
        @Override
        public void onDoubleClick() {
            for (final DoubleClickListener listener : listeners) {
                listener.onDoubleClick();
            }
        }

    }

}
