/*******************************************************************************
 * I9 TECHNOLOGIES CONFIDENTIAL
 * __________________
 * Copyright © 2013 i9 Technologies 
 * All Rights Reserved.
 * 
 * NOTICE:  All information contained herein is, and remains the property 
 * of i9 Technologies and its suppliers, if any.  The intellectual and technical 
 * concepts contained herein are proprietary to i9 Technologies and its 
 * suppliers and may be covered by U.S. and Foreign Patents, patents in 
 * process, and are protected by trade secret or copyright law. Dissemination 
 * of this information or reproduction of this material is strictly forbidden 
 * unless prior written permission is obtained from i9 Technologies.
 ******************************************************************************/
package com.xmlnamespace.panel.vaadin.ui;

import com.vaadin.event.DataBoundTransferable;
import com.vaadin.event.dd.DragAndDropEvent;
import com.vaadin.event.dd.DropHandler;
import com.vaadin.event.dd.acceptcriteria.AcceptCriterion;
import com.vaadin.event.dd.acceptcriteria.ServerSideCriterion;
import com.vaadin.ui.DragAndDropWrapper;

public abstract class DragAndDropComponent extends DragAndDropWrapper implements DropHandler {

	private Object dragObject;
	
	private Class<?>[] acceptClasses;
	
	private static final long serialVersionUID = 1L;

	public DragAndDropComponent(Class<?>[] acceptClasses, Object dragObject) {
		super(null);
		setAcceptClasses(acceptClasses);
		setDragObject(dragObject);
	}

	public void setDragObject(Object dragObject) {
		this.dragObject = dragObject;
		
		//If drag object is not null enable drag else disable it
		if(dragObject != null) {
			setDragStartMode(DragStartMode.WRAPPER);
		} else {
			setDragStartMode(DragStartMode.NONE);
		}
	}
	
	public Object getDragObject() {
		return dragObject;
	}
	
	public Class<?>[] getAcceptClasses() {
		return acceptClasses;
	}

	public void setAcceptClasses(Class<?>[] acceptClasses) {
		this.acceptClasses = acceptClasses;
		
		//If accepted classes is not null enable drop else disable it
		if(acceptClasses != null) {
			setDropHandler(this);
		} else {
			setDropHandler(null);
		}
	}

	@Override
	public void drop(DragAndDropEvent event) {
		handleDrop(event, getDataFromDragAndDropEvent(event));
	}

	protected void handleDrop(DragAndDropEvent event, Object object) {
		//Do nothing. If sub classes wish to handle drop events the need to override this method
	}
	
	@Override
	public AcceptCriterion getAcceptCriterion() {
		return new ClassTypeCriterion();
	}

	private static Object getDataFromDragAndDropEvent(DragAndDropEvent event) {
		
		//If drop is a DragAndDropComponent grab it from the custom method else if it is a DataBoundTransferable use the get item id class
		if(event.getTransferable().getSourceComponent() instanceof DragAndDropComponent) {
			return ((DragAndDropComponent)event.getTransferable().getSourceComponent()).getDragObject();
		} else if(event.getTransferable() instanceof DataBoundTransferable) {
			return ((DataBoundTransferable)event.getTransferable()).getItemId();
		} else if(event.getTransferable() instanceof UserComponent.UserComponentTransferable) {
			return ((UserComponent.UserComponentTransferable)event.getTransferable()).getObject();
		} else if(event.getTransferable() instanceof CallComponent.CallComponentTransferable) {
		        return ((CallComponent.CallComponentTransferable)event.getTransferable()).getObject();
		} else {
			throw new IllegalArgumentException("Drop type not known");
		}	
	}
	
	private class ClassTypeCriterion extends ServerSideCriterion {

		private static final long serialVersionUID = 1L;
		
		@Override
		public boolean accept(DragAndDropEvent dragEvent) {
			
			//Check if the destination is the same as the source if so don't accept
			if(dragEvent.getTransferable().getSourceComponent().equals(DragAndDropComponent.this)) {
				return false;
			}
						
			//If accept classes are set check if any match if so accept
			if(acceptClasses != null) {
				Object data = null;
				try {
					data = getDataFromDragAndDropEvent(dragEvent);
				} catch (Exception e) {
					return false;
				}

				for(Class<?> classType : acceptClasses) {
					if(classType.isAssignableFrom(data.getClass())) {
						return true;
					}
				}
			}
			
			return false;
		}
	}
	
}
